.harddiskled-label {
    font-size: 1em;
    min-width: 4em;
    text-align: right;
}

.harddiskled-icon {
    font-size: 1em;
    min-width: 1.2em;
    text-align: center;
    color: lime;
}

.harddiskled-container {
    spacing: 0px;
    margin-left: 2px;
    margin-right: 2px;
}
